@@ -26,6 +26,7 @@
 #include "sync-httpd_mhd.h"
 #include "sync_database_lib.h"
 #include "sync-httpd_backup.h"
+#include "sync-httpd_config.h"
 #include "sync-httpd_terms.h"
 
 /**
@@ -90,24 +91,6 @@ static struct MHD_Daemon *mhd;
 struct SYNC_DatabasePlugin *db;
 
 
-/**
- * Return #GNUNET_YES if given a valid correlation ID and
- * #GNUNET_NO otherwise.
- *
- * @returns #GNUNET_YES iff given a valid correlation ID
- */
-static int
-is_valid_correlation_id (const char *correlation_id)
-{
-  if (strlen (correlation_id) >= 64)
-    return GNUNET_NO;
-  for (size_t i = 0; i < strlen (correlation_id); i++)
-    if (! (isalnum (correlation_id[i]) || (correlation_id[i] == '-')))
-      return GNUNET_NO;
-  return GNUNET_YES;
-}
-
-
 /**
  * A client has requested the given url using the given method
  * (#MHD_HTTP_METHOD_GET, #MHD_HTTP_METHOD_PUT,
@@ -144,10 +127,10 @@ is_valid_correlation_id (const char *correlation_id)
  *        can be set with the #MHD_OPTION_NOTIFY_COMPLETED).
  *        Initially, `*con_cls` will be NULL.
  * @return #MHD_YES if the connection was handled successfully,
- *         #MHD_NO if the socket must be closed due to a serios
+ *         #MHD_NO if the socket must be closed due to a serious
  *         error while handling the request
  */
-static int
+static MHD_RESULT
 url_handler (void *cls,
              struct MHD_Connection *connection,
              const char *url,
@@ -164,7 +147,10 @@ url_handler (void *cls,
       &SH_MHD_handler_static_response, MHD_HTTP_OK },
     { "/agpl", MHD_HTTP_METHOD_GET, "text/plain",
       NULL, 0,
-      &SH_MHD_handler_agpl_redirect, MHD_HTTP_FOUND },
+      &SH_handler_config, MHD_HTTP_FOUND },
+    { "/config", MHD_HTTP_METHOD_GET, "text/json",
+      NULL, 0,
+      &SH_handler_terms, MHD_HTTP_OK },
     { "/terms", MHD_HTTP_METHOD_GET, "text/plain",
       NULL, 0,
       &SH_handler_terms, MHD_HTTP_OK },
@@ -189,7 +175,6 @@ url_handler (void *cls,
   (void) cls;
   (void) version;
   hc = *con_cls;
-
   if (NULL == hc)
   {
     GNUNET_async_scope_fresh (&aid);
@@ -198,7 +183,7 @@ url_handler (void *cls,
                                                   MHD_HEADER_KIND,
                                                   "Sync-Correlation-Id");
     if ((NULL != correlation_id) &&
-        (GNUNET_YES != is_valid_correlation_id (correlation_id)))
+        (GNUNET_YES != GNUNET_CURL_is_valid_scope_id (correlation_id)))
     {
       GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                   "illegal incoming correlation ID\n");
@@ -240,6 +225,11 @@ url_handler (void *cls,
                                              upload_data,
                                              upload_data_size);
     }
+    if (0 == strcasecmp (method,
+                         MHD_HTTP_METHOD_OPTIONS))
+    {
+      return TALER_MHD_reply_cors_preflight (connection);
+    }
     if (0 == strcasecmp (method,
                          MHD_HTTP_METHOD_GET))
     {
@@ -260,7 +250,7 @@ url_handler (void *cls,
       if (NULL != hc)
       {
         /* Store the async context ID, so we can restore it if
-         * we get another callack for this request. */
+         * we get another callback for this request. */
         hc->async_scope_id = aid;
       }
       return ret;
@@ -270,27 +260,34 @@ url_handler (void *cls,
   {
     struct SH_RequestHandler *rh = &handlers[i];
 
-    if ( (0 == strcmp (url,
-                       rh->url)) &&
-         ( (NULL == rh->method) ||
-           (0 == strcasecmp (method,
-                             rh->method)) ) )
+    if (0 == strcmp (url,
+                     rh->url))
     {
-      int ret;
-
-      ret = rh->handler (rh,
-                         connection,
-                         con_cls,
-                         upload_data,
-                         upload_data_size);
-      hc = *con_cls;
-      if (NULL != hc)
+      if (0 == strcasecmp (method,
+                           MHD_HTTP_METHOD_OPTIONS))
       {
-        /* Store the async context ID, so we can restore it if
-         * we get another callack for this request. */
-        hc->async_scope_id = aid;
+        return TALER_MHD_reply_cors_preflight (connection);
+      }
+      if ( (NULL == rh->method) ||
+           (0 == strcasecmp (method,
+                             rh->method)) )
+      {
+        MHD_RESULT ret;
+
+        ret = rh->handler (rh,
+                           connection,
+                           con_cls,
+                           upload_data,
+                           upload_data_size);
+        hc = *con_cls;
+        if (NULL != hc)
+        {
+          /* Store the async context ID, so we can restore it if
+           * we get another callback for this request. */
+          hc->async_scope_id = aid;
+        }
+        return ret;
       }
-      return ret;
     }
   }
   return SH_MHD_handler_static_response (&h404,
@@ -311,6 +308,12 @@ static void
 do_shutdown (void *cls)
 {
   (void) cls;
+  SH_resume_all_bc ();
+  if (NULL != mhd_task)
+  {
+    GNUNET_SCHEDULER_cancel (mhd_task);
+    mhd_task = NULL;
+  }
   if (NULL != SH_ctx)
   {
     GNUNET_CURL_fini (SH_ctx);
@@ -321,12 +324,6 @@ do_shutdown (void *cls)
     GNUNET_CURL_gnunet_rc_destroy (rc);
     rc = NULL;
   }
-  SH_resume_all_bc ();
-  if (NULL != mhd_task)
-  {
-    GNUNET_SCHEDULER_cancel (mhd_task);
-    mhd_task = NULL;
-  }
   if (NULL != mhd)
   {
     MHD_stop_daemon (mhd);
@@ -537,10 +534,10 @@ run (void *cls,
     return;
   }
   if (GNUNET_OK !=
-      TALER_config_get_denom (config,
-                              "sync",
-                              "ANNUAL_FEE",
-                              &SH_annual_fee))
+      TALER_config_get_amount (config,
+                               "sync",
+                               "ANNUAL_FEE",
+                               &SH_annual_fee))
   {
     GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
                                "sync",
@@ -633,7 +630,6 @@ main (int argc,
                                "connection-close",
                                "force HTTP connections to be closed after each request",
                                &SH_sync_connection_close),
-
     GNUNET_GETOPT_OPTION_END
   };
 
@@ -646,7 +642,8 @@ main (int argc,
       GNUNET_PROGRAM_run (argc, argv,
                           "sync-httpd",
                           "sync HTTP interface",
-                          options, &run, NULL))
+                          options,
+                          &run, NULL))
     return 3;
   return (GNUNET_OK == result) ? 0 : 1;
 }
