@@ -67,6 +67,11 @@ struct BackupContext
    */
   struct GNUNET_HashCode new_backup_hash;
 
+  /**
+   * Claim token, all zeros if not known. Only set if @e existing_order_id is non-NULL.
+   */
+  struct TALER_ClaimTokenP token;
+
   /**
    * Hash context for the upload.
    */
@@ -95,12 +100,12 @@ struct BackupContext
   /**
    * Used while we are awaiting proposal creation.
    */
-  struct TALER_MERCHANT_ProposalOperation *po;
+  struct TALER_MERCHANT_PostOrdersOperation *po;
 
   /**
    * Used while we are waiting payment.
    */
-  struct TALER_MERCHANT_CheckPaymentOperation *cpo;
+  struct TALER_MERCHANT_OrderMerchantGetHandle *omgh;
 
   /**
    * HTTP response code to use on resume, if non-NULL.
@@ -168,13 +173,13 @@ SH_resume_all_bc ()
     MHD_resume_connection (bc->con);
     if (NULL != bc->po)
     {
-      TALER_MERCHANT_proposal_cancel (bc->po);
+      TALER_MERCHANT_orders_post_cancel (bc->po);
       bc->po = NULL;
     }
-    if (NULL != bc->cpo)
+    if (NULL != bc->omgh)
     {
-      TALER_MERCHANT_check_payment_cancel (bc->cpo);
-      bc->cpo = NULL;
+      TALER_MERCHANT_merchant_order_get_cancel (bc->omgh);
+      bc->omgh = NULL;
     }
   }
 }
@@ -191,13 +196,13 @@ cleanup_ctx (struct TM_HandlerContext *hc)
   struct BackupContext *bc = (struct BackupContext *) hc;
 
   if (NULL != bc->po)
-    TALER_MERCHANT_proposal_cancel (bc->po);
+    TALER_MERCHANT_orders_post_cancel (bc->po);
   if (NULL != bc->hash_ctx)
     GNUNET_CRYPTO_hash_context_abort (bc->hash_ctx);
   if (NULL != bc->resp)
     MHD_destroy_response (bc->resp);
-  GNUNET_free_non_null (bc->existing_order_id);
-  GNUNET_free_non_null (bc->upload);
+  GNUNET_free (bc->existing_order_id);
+  GNUNET_free (bc->upload);
   GNUNET_free (bc);
 }
 
@@ -207,10 +212,13 @@ cleanup_ctx (struct TM_HandlerContext *hc)
  *
  * @param connection MHD connection
  * @param order_id our backend's order ID
- * @return MHD repsonse to use
+ * @param token the claim token generated by the merchant (NULL if
+ *        it wasn't generated).
+ * @return MHD response to use
  */
 static struct MHD_Response *
-make_payment_request (const char *order_id)
+make_payment_request (const char *order_id,
+                      const struct TALER_ClaimTokenP *token)
 {
   struct MHD_Response *resp;
 
@@ -221,18 +229,67 @@ make_payment_request (const char *order_id)
   resp = MHD_create_response_from_buffer (0,
                                           NULL,
                                           MHD_RESPMEM_PERSISTENT);
-  GNUNET_break (MHD_YES ==
-                MHD_add_response_header (resp,
-                                         MHD_HTTP_HEADER_ACCESS_CONTROL_ALLOW_ORIGIN,
-                                         "*"));
+  TALER_MHD_add_global_headers (resp);
   {
     char *hdr;
+    char *pfx;
+    char *hn;
 
-    /* TODO: support instances? */
-    GNUNET_asprintf (&hdr,
-                     "taler://pay/%s/-/-/%s",
-                     SH_backend_url,
-                     order_id);
+    if (0 == strncasecmp ("https://",
+                          SH_backend_url,
+                          strlen ("https://")))
+    {
+      pfx = "taler://";
+      hn = &SH_backend_url[strlen ("https://")];
+    }
+    else if (0 == strncasecmp ("http://",
+                               SH_backend_url,
+                               strlen ("http://")))
+    {
+      pfx = "taler+http://";
+      hn = &SH_backend_url[strlen ("http://")];
+    }
+    else
+    {
+      GNUNET_break (0);
+      MHD_destroy_response (resp);
+      return NULL;
+    }
+    if (0 == strlen (hn))
+    {
+      GNUNET_break (0);
+      MHD_destroy_response (resp);
+      return NULL;
+    }
+    // FIXME: support instances?
+    if (NULL != token)
+    {
+      char tok[256];
+
+      /* This path should not be taken, as we disabled tokens */
+      GNUNET_assert (NULL !=
+                     GNUNET_STRINGS_data_to_string (token,
+                                                    sizeof (*token),
+                                                    tok,
+                                                    sizeof (tok)));
+      GNUNET_asprintf (&hdr,
+                       "%spay/%s%s%s/?c=%s",
+                       pfx,
+                       hn,
+                       ('/' == hn[strlen (hn) - 1])
+                       ? ""
+                       : "/",
+                       order_id,
+                       tok);
+    }
+    else
+    {
+      GNUNET_asprintf (&hdr,
+                       "%spay/%s/%s/",
+                       pfx,
+                       hn,
+                       order_id);
+    }
     GNUNET_break (MHD_YES ==
                   MHD_add_response_header (resp,
                                            "Taler",
@@ -248,18 +305,16 @@ make_payment_request (const char *order_id)
  * /contract request to a merchant.
  *
  * @param cls our `struct BackupContext`
- * @param http_status HTTP response code, 200 indicates success;
- *                    0 if the backend's reply is bogus (fails to follow the protocol)
- * @param ec taler-specific error code
- * @param obj raw JSON reply, or error details if the request failed
+ * @param hr HTTP response details
  * @param order_id order id of the newly created order
+ * @param token the claim token generated by the merchant (NULL if
+ *        it wasn't generated).
  */
 static void
 proposal_cb (void *cls,
-             unsigned int http_status,
-             enum TALER_ErrorCode ec,
-             const json_t *obj,
-             const char *order_id)
+             const struct TALER_MERCHANT_HttpResponse *hr,
+             const char *order_id,
+             const struct TALER_ClaimTokenP *token)
 {
   struct BackupContext *bc = cls;
   enum SYNC_DB_QueryStatus qs;
@@ -273,24 +328,27 @@ proposal_cb (void *cls,
                                bc);
   MHD_resume_connection (bc->con);
   SH_trigger_daemon ();
-  if (MHD_HTTP_OK != http_status)
+  if (MHD_HTTP_OK != hr->http_status)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                 "Backend returned status %u/%u\n",
-                http_status,
-                (unsigned int) ec);
+                hr->http_status,
+                (unsigned int) hr->ec);
     GNUNET_break (0);
-    bc->resp = TALER_MHD_make_json_pack ("{s:I, s:s, s:I, s:I}",
+    bc->resp = TALER_MHD_make_json_pack ("{s:I, s:s, s:I, s:I, s:O}",
                                          "code",
                                          (json_int_t)
                                          TALER_EC_SYNC_PAYMENT_CREATE_BACKEND_ERROR,
                                          "hint",
                                          "Failed to setup order with merchant backend",
-                                         "backend-ec", (json_int_t) ec,
+                                         "backend-ec",
+                                         (json_int_t) hr->ec,
                                          "backend-http-status",
-                                         (json_int_t) http_status);
+                                         (json_int_t) hr->http_status,
+                                         "backend-reply",
+                                         hr->reply);
     GNUNET_assert (NULL != bc->resp);
-    bc->response_code = MHD_HTTP_INTERNAL_SERVER_ERROR;
+    bc->response_code = MHD_HTTP_FAILED_DEPENDENCY;
     return;
   }
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
@@ -299,6 +357,7 @@ proposal_cb (void *cls,
   qs = db->store_payment_TR (db->cls,
                              &bc->account,
                              order_id,
+                             token,
                              &SH_annual_fee);
   if (0 >= qs)
   {
@@ -312,7 +371,8 @@ proposal_cb (void *cls,
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Obtained fresh order `%s'\n",
               order_id);
-  bc->resp = make_payment_request (order_id);
+  bc->resp = make_payment_request (order_id,
+                                   token);
   GNUNET_assert (NULL != bc->resp);
   bc->response_code = MHD_HTTP_PAYMENT_REQUIRED;
 }
@@ -325,12 +385,14 @@ proposal_cb (void *cls,
  * @param cls closure, our `struct BackupContext`
  * @param timestamp for how long have we been waiting
  * @param order_id order id in the backend
+ * @param token claim token to use (or NULL for none)
  * @param amount how much is the order for
  */
 static void
 ongoing_payment_cb (void *cls,
                     struct GNUNET_TIME_Absolute timestamp,
                     const char *order_id,
+                    const struct TALER_ClaimTokenP *token,
                     const struct TALER_Amount *amount)
 {
   struct BackupContext *bc = cls;
@@ -342,9 +404,11 @@ ongoing_payment_cb (void *cls,
   if ( (NULL == bc->existing_order_id) ||
        (bc->existing_order_timestamp.abs_value_us < timestamp.abs_value_us) )
   {
-    GNUNET_free_non_null (bc->existing_order_id);
+    GNUNET_free (bc->existing_order_id);
     bc->existing_order_id = GNUNET_strdup (order_id);
     bc->existing_order_timestamp = timestamp;
+    if (NULL != token)
+      bc->token = *token;
   }
 }
 
@@ -353,64 +417,58 @@ ongoing_payment_cb (void *cls,
  * Callback to process a GET /check-payment request
  *
  * @param cls our `struct BackupContext`
- * @param http_status HTTP status code for this request
- * @param obj raw response body
- * @param paid #GNUNET_YES if the payment is settled, #GNUNET_NO if not
- *        settled, $GNUNET_SYSERR on error
- *        (note that refunded payments are returned as paid!)
- * @param refunded #GNUNET_YES if there is at least on refund on this payment,
- *        #GNUNET_NO if refunded, #GNUNET_SYSERR or error
- * @param refunded_amount amount that was refunded, NULL if there
- *        was no refund
- * @param taler_pay_uri the URI that instructs the wallets to process
- *                      the payment
+ * @param hr HTTP response details
+ * @param osr order status
  */
 static void
 check_payment_cb (void *cls,
-                  unsigned int http_status,
-                  const json_t *obj,
-                  int paid,
-                  int refunded,
-                  struct TALER_Amount *refund_amount,
-                  const char *taler_pay_uri)
+                  const struct TALER_MERCHANT_HttpResponse *hr,
+                  const struct TALER_MERCHANT_OrderStatusResponse *osr)
 {
   struct BackupContext *bc = cls;
 
   /* refunds are not supported, verify */
-  bc->cpo = NULL;
+  bc->omgh = NULL;
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Payment status checked: %s\n",
-              paid ? "paid" : "unpaid");
+              "Payment status checked: %d\n",
+              osr->status);
   GNUNET_CONTAINER_DLL_remove (bc_head,
                                bc_tail,
                                bc);
   MHD_resume_connection (bc->con);
   SH_trigger_daemon ();
-  GNUNET_break ( (GNUNET_NO == refunded) &&
-                 (NULL == refund_amount) );
-  if (paid)
+  switch (osr->status)
   {
-    enum SYNC_DB_QueryStatus qs;
+  case TALER_MERCHANT_OSC_PAID:
+    {
+      enum SYNC_DB_QueryStatus qs;
 
-    qs = db->increment_lifetime_TR (db->cls,
-                                    &bc->account,
-                                    bc->order_id,
-                                    GNUNET_TIME_UNIT_YEARS); /* always annual */
-    if (0 <= qs)
+      qs = db->increment_lifetime_TR (db->cls,
+                                      &bc->account,
+                                      bc->order_id,
+                                      GNUNET_TIME_UNIT_YEARS); /* always annual */
+      if (0 <= qs)
+        return; /* continue as planned */
+      GNUNET_break (0);
+      bc->resp = TALER_MHD_make_error (TALER_EC_SYNC_PAYMENT_CONFIRM_DB_ERROR,
+                                       "Failed to persist payment confirmation in sync database");
+      GNUNET_assert (NULL != bc->resp);
+      bc->response_code = MHD_HTTP_INTERNAL_SERVER_ERROR;
       return; /* continue as planned */
-    GNUNET_break (0);
-    bc->resp = TALER_MHD_make_error (TALER_EC_SYNC_PAYMENT_CONFIRM_DB_ERROR,
-                                     "Failed to persist payment confirmation in sync database");
-    GNUNET_assert (NULL != bc->resp);
-    bc->response_code = MHD_HTTP_INTERNAL_SERVER_ERROR;
-    return; /* continue as planned */
+    }
+  case TALER_MERCHANT_OSC_UNPAID:
+  case TALER_MERCHANT_OSC_CLAIMED:
+    break;
   }
   if (NULL != bc->existing_order_id)
   {
     /* repeat payment request */
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Repeating payment request\n");
-    bc->resp = make_payment_request (bc->existing_order_id);
+    bc->resp = make_payment_request (bc->existing_order_id,
+                                     (GNUNET_YES == GNUNET_is_zero (&bc->token))
+                                     ? NULL
+                                     : &bc->token);
     GNUNET_assert (NULL != bc->resp);
     bc->response_code = MHD_HTTP_PAYMENT_REQUIRED;
     return;
@@ -442,13 +500,14 @@ await_payment (struct BackupContext *bc,
                                bc);
   MHD_suspend_connection (bc->con);
   bc->order_id = order_id;
-  bc->cpo = TALER_MERCHANT_check_payment (SH_ctx,
-                                          SH_backend_url,
-                                          order_id,
-                                          NULL /* our payments are NOT session-bound */,
-                                          timeout,
-                                          &check_payment_cb,
-                                          bc);
+  bc->omgh = TALER_MERCHANT_merchant_order_get (SH_ctx,
+                                                SH_backend_url,
+                                                order_id,
+                                                NULL /* our payments are NOT session-bound */,
+                                                false,
+                                                timeout,
+                                                &check_payment_cb,
+                                                bc);
   SH_trigger_curl ();
 }
 
@@ -464,7 +523,7 @@ await_payment (struct BackupContext *bc,
  *                #GNUNET_NO if payment is needed
  * @return MHD status code
  */
-static int
+static MHD_RESULT
 begin_payment (struct BackupContext *bc,
                int pay_req)
 {
@@ -478,7 +537,7 @@ begin_payment (struct BackupContext *bc,
   if (qs < 0)
   {
     struct MHD_Response *resp;
-    int ret;
+    MHD_RESULT ret;
 
     resp = TALER_MHD_make_error (TALER_EC_SYNC_PAYMENT_CHECK_ORDER_DB_ERROR,
                                  "Failed to check for existing orders in sync database");
@@ -510,11 +569,18 @@ begin_payment (struct BackupContext *bc,
                      "amount", TALER_JSON_from_amount (&SH_annual_fee),
                      "summary", "annual fee for sync service",
                      "fulfillment_url", SH_fulfillment_url);
-  bc->po = TALER_MERCHANT_order_put (SH_ctx,
-                                     SH_backend_url,
-                                     order,
-                                     &proposal_cb,
-                                     bc);
+  bc->po = TALER_MERCHANT_orders_post2 (SH_ctx,
+                                        SH_backend_url,
+                                        order,
+                                        GNUNET_TIME_UNIT_ZERO,
+                                        NULL, /* no payment target */
+                                        0,
+                                        NULL, /* no inventory products */
+                                        0,
+                                        NULL, /* no uuids */
+                                        false, /* do NOT require claim token */
+                                        &proposal_cb,
+                                        bc);
   SH_trigger_curl ();
   json_decref (order);
   return MHD_YES;
@@ -530,7 +596,7 @@ begin_payment (struct BackupContext *bc,
  * @param qs query status to handle
  * @return #MHD_YES or #MHD_NO
  */
-static int
+static MHD_RESULT
 handle_database_error (struct BackupContext *bc,
                        enum SYNC_DB_QueryStatus qs)
 {
@@ -542,8 +608,8 @@ handle_database_error (struct BackupContext *bc,
     return TALER_MHD_reply_with_error (bc->con,
                                        MHD_HTTP_NOT_FOUND,
                                        TALER_EC_SYNC_PREVIOUS_BACKUP_UNKNOWN,
-                                       "Cannot update, no existing backup known");
-  case SYNC_DB_OLD_BACKUP_MISSMATCH:
+                                       NULL);
+  case SYNC_DB_OLD_BACKUP_MISMATCH:
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Conflict detected, returning existing backup\n");
     return SH_return_backup (bc->con,
@@ -577,7 +643,7 @@ handle_database_error (struct BackupContext *bc,
     return TALER_MHD_reply_with_error (bc->con,
                                        MHD_HTTP_INTERNAL_SERVER_ERROR,
                                        TALER_EC_SYNC_DATABASE_FETCH_ERROR,
-                                       "failed to fetch existing record from database");
+                                       NULL);
   case SYNC_DB_NO_RESULTS:
     GNUNET_assert (0);
     return MHD_NO;
@@ -601,7 +667,7 @@ handle_database_error (struct BackupContext *bc,
  * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
  * @return MHD result code
  */
-int
+MHD_RESULT
 SH_backup_post (struct MHD_Connection *connection,
                 void **con_cls,
                 const struct SYNC_AccountPublicKeyP *account,
@@ -635,20 +701,21 @@ SH_backup_post (struct MHD_Connection *connection,
                     &len)) )
       {
         GNUNET_break_op (0);
-        return TALER_MHD_reply_with_error (connection,
-                                           MHD_HTTP_BAD_REQUEST,
-                                           TALER_EC_SYNC_BAD_CONTENT_LENGTH,
-                                           (NULL == lens)
-                                           ? "Content-length value missing"
-                                           : "Content-length value malformed");
+        return TALER_MHD_reply_with_error (
+          connection,
+          MHD_HTTP_BAD_REQUEST,
+          (NULL == lens)
+          ? TALER_EC_SYNC_MALFORMED_CONTENT_LENGTH
+          : TALER_EC_SYNC_MISSING_CONTENT_LENGTH,
+          lens);
       }
       if (len / 1024 / 1024 >= SH_upload_limit_mb)
       {
         GNUNET_break_op (0);
         return TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_PAYLOAD_TOO_LARGE,
-                                           TALER_EC_SYNC_BAD_CONTENT_LENGTH,
-                                           "Content-length value not acceptable");
+                                           TALER_EC_SYNC_EXCESSIVE_CONTENT_LENGTH,
+                                           NULL);
       }
       bc->upload = GNUNET_malloc_large (len);
       if (NULL == bc->upload)
@@ -658,7 +725,7 @@ SH_backup_post (struct MHD_Connection *connection,
         return TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_PAYLOAD_TOO_LARGE,
                                            TALER_EC_SYNC_OUT_OF_MEMORY_ON_CONTENT_LENGTH,
-                                           "Server out of memory, try again later");
+                                           NULL);
       }
       bc->upload_size = (size_t) len;
     }
@@ -679,7 +746,7 @@ SH_backup_post (struct MHD_Connection *connection,
         return TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_BAD_REQUEST,
                                            TALER_EC_SYNC_BAD_IF_MATCH,
-                                           "If-Match does not include not a base32-encoded SHA-512 hash");
+                                           NULL);
       }
     }
     {
@@ -699,7 +766,7 @@ SH_backup_post (struct MHD_Connection *connection,
         return TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_BAD_REQUEST,
                                            TALER_EC_SYNC_BAD_SYNC_SIGNATURE,
-                                           "Sync-Signature does not include a base32-encoded EdDSA signature");
+                                           NULL);
       }
     }
     {
@@ -719,20 +786,21 @@ SH_backup_post (struct MHD_Connection *connection,
         return TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_BAD_REQUEST,
                                            TALER_EC_SYNC_BAD_IF_NONE_MATCH,
-                                           "If-none-match does not include not a base32-encoded SHA-512 hash");
+                                           NULL);
       }
     }
     /* validate signature */
     {
-      struct SYNC_UploadSignaturePS usp;
+      struct SYNC_UploadSignaturePS usp = {
+        .purpose.size = htonl (sizeof (usp)),
+        .purpose.purpose = htonl (TALER_SIGNATURE_SYNC_BACKUP_UPLOAD),
+        .old_backup_hash = bc->old_backup_hash,
+        .new_backup_hash = bc->new_backup_hash
+      };
 
-      usp.purpose.size = htonl (sizeof (struct SYNC_UploadSignaturePS));
-      usp.purpose.purpose = htonl (TALER_SIGNATURE_SYNC_BACKUP_UPLOAD);
-      usp.old_backup_hash = bc->old_backup_hash;
-      usp.new_backup_hash = bc->new_backup_hash;
       if (GNUNET_OK !=
           GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_SYNC_BACKUP_UPLOAD,
-                                      &usp.purpose,
+                                      &usp,
                                       &bc->account_sig.eddsa_sig,
                                       &account->eddsa_pub))
       {
@@ -740,13 +808,13 @@ SH_backup_post (struct MHD_Connection *connection,
         return TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_FORBIDDEN,
                                            TALER_EC_SYNC_INVALID_SIGNATURE,
-                                           "Account signature does not match upload");
+                                           NULL);
       }
     }
     /* get ready to hash (done here as we may go async for payments next) */
     bc->hash_ctx = GNUNET_CRYPTO_hash_context_start ();
 
-    /* Check database to see if the transaction is permissable */
+    /* Check database to see if the transaction is permissible */
     {
       struct GNUNET_HashCode hc;
       enum SYNC_DB_QueryStatus qs;
@@ -764,15 +832,12 @@ SH_backup_post (struct MHD_Connection *connection,
       {
         /* Refuse upload: we already have that backup! */
         struct MHD_Response *resp;
-        int ret;
+        MHD_RESULT ret;
 
         resp = MHD_create_response_from_buffer (0,
                                                 NULL,
                                                 MHD_RESPMEM_PERSISTENT);
-        GNUNET_break (MHD_YES ==
-                      MHD_add_response_header (resp,
-                                               MHD_HTTP_HEADER_ACCESS_CONTROL_ALLOW_ORIGIN,
-                                               "*"));
+        TALER_MHD_add_global_headers (resp);
         ret = MHD_queue_response (connection,
                                   MHD_HTTP_NOT_MODIFIED,
                                   resp);
@@ -836,7 +901,7 @@ SH_backup_post (struct MHD_Connection *connection,
   }
   if (NULL != bc->resp)
   {
-    int ret;
+    MHD_RESULT ret;
 
     /* We generated a response asynchronously, queue that */
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
@@ -865,7 +930,7 @@ SH_backup_post (struct MHD_Connection *connection,
       return TALER_MHD_reply_with_error (connection,
                                          MHD_HTTP_BAD_REQUEST,
                                          TALER_EC_SYNC_INVALID_UPLOAD,
-                                         "Data uploaded does not match Etag promise");
+                                         NULL);
     }
   }
 
@@ -873,7 +938,7 @@ SH_backup_post (struct MHD_Connection *connection,
   {
     enum SYNC_DB_QueryStatus qs;
 
-    if (0 == GNUNET_is_zero (&bc->old_backup_hash))
+    if (GNUNET_YES == GNUNET_is_zero (&bc->old_backup_hash))
     {
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                   "Uploading first backup to account\n");
@@ -905,15 +970,12 @@ SH_backup_post (struct MHD_Connection *connection,
          theoretically happen if another equivalent upload succeeded
          since we last checked!) */
       struct MHD_Response *resp;
-      int ret;
+      MHD_RESULT ret;
 
       resp = MHD_create_response_from_buffer (0,
                                               NULL,
                                               MHD_RESPMEM_PERSISTENT);
-      GNUNET_break (MHD_YES ==
-                    MHD_add_response_header (resp,
-                                             MHD_HTTP_HEADER_ACCESS_CONTROL_ALLOW_ORIGIN,
-                                             "*"));
+      TALER_MHD_add_global_headers (resp);
       ret = MHD_queue_response (connection,
                                 MHD_HTTP_NOT_MODIFIED,
                                 resp);
@@ -926,15 +988,12 @@ SH_backup_post (struct MHD_Connection *connection,
   /* generate main (204) standard success reply */
   {
     struct MHD_Response *resp;
-    int ret;
+    MHD_RESULT ret;
 
     resp = MHD_create_response_from_buffer (0,
                                             NULL,
                                             MHD_RESPMEM_PERSISTENT);
-    GNUNET_break (MHD_YES ==
-                  MHD_add_response_header (resp,
-                                           MHD_HTTP_HEADER_ACCESS_CONTROL_ALLOW_ORIGIN,
-                                           "*"));
+    TALER_MHD_add_global_headers (resp);
     ret = MHD_queue_response (connection,
                               MHD_HTTP_NO_CONTENT,
                               resp);
