@@ -33,7 +33,7 @@
  * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
  * @return MHD result code
  */
-int
+MHD_RESULT
 SH_handler_terms (struct SH_RequestHandler *rh,
                   struct MHD_Connection *connection,
                   void **connection_cls,
